/*
    Working sketch!!!

    Z21 Ethernet Emulation für die App-Steuerung via Smartphone über XpressNet.

  My IP adress of EthernetShield ("z21") is 192.168.0.112
  You can put  yours, depends on your LAN configuration

  Hardware:
  - Arduino Leonardo ETH;
  - Ethernet Shield W5500
  - Protoshield V5 with mini breadboard
  - electronic components:

  MAX485 RX/TX are connected to Serial1 pins of Leonardo ETH (0/1) => you can use Serial0 for debug log in Serial Terminal window of Arduino IDE app.
  Control PIN is pin 3.

  LED on pin 13 is indicator of XpressNet connection: flashes - no connection, solid - OK.

  Plug XpressNet cable from z21 emulator to Slave socket of your ROCO command station. Your Multimouse you should plug as Master. 

  Current ISSUES(to do):
  - no indication of actions on z21 app. 
  - no indication of MultiMouse actions on z21 app

  */

// connect XpressNet lib
#include <XpressNet.h>

XpressNetClass XpressNet;

#include <EEPROM.h>
#define EEip 10    //Startddress im EEPROM für die IP
#define EEXNet 9   //Adresse im XNet-Bus

#include <SPI.h>         // needed for Arduino versions later than 0018
#include <Ethernet2.h>
#include <EthernetUdp2.h>         // UDP library

#include "z21.h" // connect z21 library. Its should be in the same zip file with whole project. 

// Enter a MAC address and IP address for your controller below.
// The IP address will be dependent on your local network:

// Define MAC address of your Ethernet shield:
byte mac[] = { 0x90, 0xA2, 0xDA, 0x0F, 0xFE, 0xDC }; // Check with your W5100 shield MAC address!!!

// IP address of z21. Depends on your LAN configuration. Should be the same sub net.
IPAddress ip(192, 168, 0, 112);

//Run ethernet shield in server mode
EthernetServer server(80);

#define ResetPin A1  //Reset Pin bei Neustart betätigen um Standard IP zu setzten!

// XpressNet address: must be in range of 1-31; must be unique. Note that some IDs
// are currently used by default, like 2 for a LH90 or LH100 out of the box, or 30
// for PC interface devices like the XnTCP.
byte XNetAddress = 29;    //Adresse im XpressNet

#define interval 2000   //interval at milliseconds

long previousMillis = 0;        // will store last time of IP decount updated


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// S E T U P


void setup() {

  Serial.begin(9600); // Debug via terminal window at 9600

  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }

  Serial.println("Z21");

  pinMode(ResetPin, INPUT); //define reset button pin as input pin
  digitalWrite(ResetPin, HIGH);  //PullUp to HIGH
  delay(100);

 
  // start the Ethernet and UDP:
  Ethernet.begin(mac, ip);

  Serial.print("Starting Ethernet at IP address:");
  Serial.println(ip);

  Serial.println("Starting XPressNet");
  XpressNet.start(XNetAddress, 3);   // Start XpressNet lib with XnetAddress and PIN 3 as control pin. If you use Mega, then XpressNet lin use Serial1 for MAX485 communication

  Serial.println("Starting Z21 Emulation");

  z21Setup();

  Serial.println("Setup finished");
}

// end of SETUP

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// L O O P


void loop() {

  XpressNet.receive();  // XpressNet work funk: read and write to X-net bus. 

  z21Receive();    //Read and wright data to z21 apps. 

  XpressNet.receive();  //XpressNet work funk: read and write to X-net bus. 

  Webconfig();    //Webserver for Configuration of IP. http://192.168.0.2 (for my IP!) 

  //Check active IP every 2 seconds.  
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis > interval) {
    previousMillis = currentMillis;
    z21CheckActiveIP();
  }
}

// end of L O O P

// Functions part:

//--------------------------------------------------------------------------------------------

// WEB server realization to change IP adress. 
void Webconfig() {
  EthernetClient client = server.available();
  if (client) {
    String receivedText = String(50);
    // an http request ends with a blank line
    boolean currentLineIsBlank = true;
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();
        if (receivedText.length() < 50) {
          receivedText += c;
        }
        // if you've gotten to the end of the line (received a newline
        // character) and the line is blank, the http request has ended,
        // so you can send a reply
        if (c == '\n' && currentLineIsBlank) {
          // send a standard http response header
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          //client.println("Connection: close");  // the connection will be closed after completion of the response
          //client.println("Refresh: 5");  // refresh the page automatically every 5 sec
          client.println();
          //Website:
          client.println("<html><head><title>Z21</title></head><body>");
          client.println("<h1>Z21</h1><br />");
          //----------------------------------------------------------------------------------------------------
          int firstPos = receivedText.indexOf("?");
          if (firstPos > -1) {
            client.println("-> accept changes after RESET!");
            byte lastPos = receivedText.indexOf(" ", firstPos);
            String theText = receivedText.substring(firstPos + 3, lastPos); // 10 is the length of "?A="
            byte XNetPos = theText.indexOf("&XNet=");
            XNetAddress = theText.substring(XNetPos + 6, theText.length()).toInt();
            byte Aip = theText.indexOf("&B=");
            byte Bip = theText.indexOf("&C=", Aip);
            byte Cip = theText.indexOf("&D=", Bip);
            byte Dip = theText.substring(Cip + 3, XNetPos).toInt();
            Cip = theText.substring(Bip + 3, Cip).toInt();
            Bip = theText.substring(Aip + 3, Bip).toInt();
            Aip = theText.substring(0, Aip).toInt();
            ip[0] = Aip;
            ip[1] = Bip;
            ip[2] = Cip;
            ip[3] = Dip;
            if (EEPROM.read(EEXNet) != XNetAddress)
              EEPROM.write(EEXNet, XNetAddress);
            if (EEPROM.read(EEip) != Aip)
              EEPROM.write(EEip, Aip);
            if (EEPROM.read(EEip + 1) != Bip)
              EEPROM.write(EEip + 1, Bip);
            if (EEPROM.read(EEip + 2) != Cip)
              EEPROM.write(EEip + 2, Cip);
            if (EEPROM.read(EEip + 3) != Dip)
              EEPROM.write(EEip + 3, Dip);
          }
          //----------------------------------------------------------------------------------------------------
          client.print("<form method=get>IP-Adr.: <input type=number min=10 max=254 name=A value=");
          client.println(ip[0]);
          client.print(">.<input type=number min=0 max=254 name=B value=");
          client.println(ip[1]);
          client.print(">.<input type=number min=0 max=254 name=C value=");
          client.println(ip[2]);
          client.print(">.<input type=number min=0 max=254 name=D value=");
          client.println(ip[3]);
          client.print("><br /> XBus Adr.: <input type=number min=1 max=31 name=XNet value=");
          client.print(XNetAddress);
          client.println("><br /><br />");
          client.println("<input type=submit></form>");
          client.println("</body></html>");
          break;
        }
        if (c == '\n')
          currentLineIsBlank = true; // you're starting a new line
        else if (c != '\r')
          currentLineIsBlank = false; // you've gotten a character on the current line
      }
    }
    client.stop();  // close the connection:
  }
} // End of Webconfig()

